@@ -1,7 +1,18 @@
+import logging
+
 from django.db.models import BooleanField
 from django.urls import reverse
 
 
+def attributes(**kwargs):
+    def inner(func):
+        for k, v in kwargs.items():
+            setattr(func, k, v)
+        return func
+
+    return inner
+
+
 class Everything:
     def __contains__(self, item):
         return True
@@ -86,7 +97,10 @@ class AnnotationDescriptor:
         return getattr(obj, self.name)
 
 
-def ddb_hide(func):
+def ddb_hide(func):  # pragma: no cover
+    logging.getLogger(__name__).warning(
+        "ddb_hide is deprecated in favor of @attributes(ddb_hide=True)"
+    )
     func.ddb_hide = True
     return func
 
