@@ -141,13 +141,47 @@ def query_html(request, *, model_name="", fields=""):
 
 @admin_decorators.staff_member_required
 def query(request, *, model_name, fields="", media):
-    profiler = None
-    if media in {"profile", "pstats"}:
-        profiler = cProfile.Profile()
-        profiler.enable()
+    if media.startswith("profile") or media.startswith("pstats"):
+        if "_" in media:
+            prof_media, media = media.split("_")
+        else:
+            prof_media = media
+            media = "json"
 
-    query = Query.from_request(model_name, fields, request.GET)
-    return _data_response(request, query, media, privilaged=True, profiler=profiler)
+        profiler = cProfile.Profile()
+        try:
+            profiler.enable()
+
+            query = Query.from_request(model_name, fields, request.GET)
+            for x in _data_response(request, query, media, privilaged=True):
+                pass
+
+            profiler.disable()
+
+            if prof_media == "profile":
+                buffer = io.StringIO()
+                stats = pstats.Stats(profiler, stream=buffer)
+                stats.sort_stats("cumulative").print_stats(50)
+                stats.sort_stats("time").print_stats(50)
+                buffer.seek(0)
+                return HttpResponse(buffer, content_type="text/plain")
+            elif prof_media == "pstats":
+                buffer = io.BytesIO()
+                marshal.dump(pstats.Stats(profiler).stats, buffer)
+                buffer.seek(0)
+                response = HttpResponse(buffer, content_type="application/octet-stream")
+                response[
+                    "Content-Disposition"
+                ] = f"attachment; filename={query.model_name}-{timezone.now().isoformat()}.pstats"
+                return response
+            else:
+                raise http.Http404(f"Bad file format {prof_media} requested")
+        finally:
+            if profiler:  # pragma: no branch
+                profiler.disable()
+    else:
+        query = Query.from_request(model_name, fields, request.GET)
+        return _data_response(request, query, media, privilaged=True)
 
 
 def view(request, pk, media):
@@ -171,15 +205,15 @@ def pad(x):
 
 
 def concat(*lists):
-    return list(itertools.chain.from_iterable(lists))
+    return itertools.chain.from_iterable(lists)
 
 
 def flip_table(table):
-    return [list(x) for x in zip(*table)]
+    return (list(x) for x in zip(*table))
 
 
 def join_tables(*tables):
-    return [concat(*ts) for ts in zip(*tables)]
+    return (list(concat(*ts)) for ts in zip(*tables))
 
 
 def format_table(fields, table, spacing=0):
@@ -194,51 +228,27 @@ def format_table(fields, table, spacing=0):
 
 
 def pad_table(x, table):
-    return [pad(x) + row for row in table]
+    p = pad(x)
+    return (p + row for row in table)
 
 
-def _data_response(request, query, media, privilaged=False, profiler=None):
+class Echo:
+    def write(self, value):
+        return value
+
+
+def _data_response(request, query, media, privilaged=False):
     orm_models = get_models(request)
     if query.model_name not in orm_models:
         raise http.Http404(f"{query.model_name} does not exist")
     bound_query = BoundQuery.bind(query, orm_models)
 
-    if profiler:
-        # get the results
-        results = get_results(request, bound_query, orm_models)
-        resp = _get_query_data(bound_query) if privilaged else {}
-        resp.update(results)
+    if media == "csv":
+        results = get_results(request, bound_query, orm_models, False)
 
-        # and throw them away and stop profiling
-        del resp
-        profiler.disable()
-
-        if media == "profile":
-            buffer = io.StringIO()
-            stats = pstats.Stats(profiler, stream=buffer)
-            stats.sort_stats("cumulative").print_stats(50)
-            stats.sort_stats("time").print_stats(50)
-            buffer.seek(0)
-            return HttpResponse(buffer, content_type="text/plain")
-        elif media == "pstats":
-            buffer = io.BytesIO()
-            marshal.dump(pstats.Stats(profiler).stats, buffer)
-            buffer.seek(0)
-            response = HttpResponse(buffer, content_type="application/octet-stream")
-            response[
-                "Content-Disposition"
-            ] = f"attachment; filename={query.model_name}-{timezone.now().isoformat()}.pstats"
-            return response
-        else:
-            assert False
-    elif media == "csv":
-        results = get_results(request, bound_query, orm_models)
-        buffer = io.StringIO()
-        writer = csv.writer(buffer)
-
-        # the pivoted column headers
-        writer.writerows(
-            pad_table(
+        def csv_rows():
+            # the pivoted column headers
+            yield from pad_table(
                 len(bound_query.row_fields) - 1,
                 flip_table(
                     format_table(
@@ -248,11 +258,9 @@ def _data_response(request, query, media, privilaged=False, profiler=None):
                     )
                 ),
             )
-        )
 
-        # the row headers and data area
-        writer.writerows(
-            pad_table(
+            # the row headers and data area
+            yield from pad_table(
                 1 - len(bound_query.row_fields),
                 join_tables(
                     format_table(bound_query.row_fields, results["rows"]),
@@ -262,16 +270,17 @@ def _data_response(request, query, media, privilaged=False, profiler=None):
                     ),
                 ),
             )
-        )
 
-        buffer.seek(0)
-        response = HttpResponse(buffer, content_type="text/csv")
+        writer = csv.writer(Echo())
+        response = http.StreamingHttpResponse(
+            (writer.writerow(row) for row in csv_rows()), content_type="text/csv"
+        )
         response[
             "Content-Disposition"
         ] = f"attachment; filename={query.model_name}-{timezone.now().isoformat()}.csv"
         return response
     elif media == "json":
-        results = get_results(request, bound_query, orm_models)
+        results = get_results(request, bound_query, orm_models, True)
         resp = _get_query_data(bound_query) if privilaged else {}
         resp.update(results)
         return JsonResponse(resp)
