@@ -1,10 +1,11 @@
 import pytest
-from data_browser.admin import ViewAdmin, globals
-from data_browser.models import View
 from django.contrib import admin
 from django.contrib.admin.utils import flatten_fieldsets
 from django.contrib.auth.models import Permission, User
 
+from data_browser.admin import ViewAdmin
+from data_browser.models import View
+
 
 @pytest.fixture
 def view(admin_user):
@@ -12,39 +13,10 @@ def view(admin_user):
 
 
 def test_open_view(view, rf):
-    expected = '<a href="/data_browser/query/app.model/fa+0,fd-1,fn.html?bob__equals=fred">view</a>'
+    expected = '<a href="/data_browser/query/app.model/fa+0,fd-1,fn.html?bob__equals=fred&limit=1000">view</a>'
     assert ViewAdmin.open_view(view) == expected
 
 
-def test_public_link(view, rf, settings):
-    globals.request = rf.get("/")
-    assert ViewAdmin.public_link(view) == "N/A"
-    view.public = True
-    expected = f"http://testserver/data_browser/view/{view.public_slug}.csv"
-    assert ViewAdmin.public_link(view) == expected
-    settings.DATA_BROWSER_ALLOW_PUBLIC = False
-    assert (
-        ViewAdmin.public_link(view) == "Public URL's are disabled in Django settings."
-    )
-    globals.request = None
-
-
-def test_google_sheets_formula(view, rf, settings):
-    globals.request = rf.get("/")
-    assert ViewAdmin.google_sheets_formula(view) == "N/A"
-    view.public = True
-    expected = (
-        f'=importdata("http://testserver/data_browser/view/{view.public_slug}.csv")'
-    )
-    assert ViewAdmin.google_sheets_formula(view) == expected
-    settings.DATA_BROWSER_ALLOW_PUBLIC = False
-    assert (
-        ViewAdmin.google_sheets_formula(view)
-        == "Public URL's are disabled in Django settings."
-    )
-    globals.request = None
-
-
 def test_add_has_user(admin_client):
     user = User.objects.get()
     res = admin_client.get("/admin/data_browser/view/add/")
@@ -78,9 +50,8 @@ def get_admin_details(rf):
         view_admin = ViewAdmin(View, admin.site)
         fields = set(flatten_fieldsets(view_admin.get_fieldsets(request, obj)))
         read_only = set(view_admin.get_readonly_fields(request, obj))
-        editable = fields - read_only
-        read_only = read_only & fields
-        return editable, read_only
+        assert fields == read_only
+        return fields
 
     return helper
 
@@ -94,8 +65,8 @@ def staff_user(admin_user):
 
 class TestAdminFieldsSuperUser:
     def test_add_page_edit_everything(self, admin_user, get_admin_details):
-        editable, read_only = get_admin_details(admin_user, None)
-        assert editable == {
+        fields = get_admin_details(admin_user, None)
+        assert fields == {
             "description",
             "fields",
             "model_name",
@@ -104,18 +75,17 @@ class TestAdminFieldsSuperUser:
             "public",
             "public_slug",
             "query",
-        }
-        assert read_only == {
             "created_time",
             "google_sheets_formula",
             "id",
             "open_view",
             "public_link",
+            "limit",
         }
 
     def test_private_view_edit_everything(self, admin_user, get_admin_details, view):
-        editable, read_only = get_admin_details(admin_user, view)
-        assert editable == {
+        fields = get_admin_details(admin_user, view)
+        assert fields == {
             "description",
             "fields",
             "model_name",
@@ -124,19 +94,18 @@ class TestAdminFieldsSuperUser:
             "public",
             "public_slug",
             "query",
-        }
-        assert read_only == {
             "created_time",
             "google_sheets_formula",
             "id",
             "open_view",
             "public_link",
+            "limit",
         }
 
     def test_public_view_edit_everything(self, admin_user, get_admin_details, view):
         view.public = True
-        editable, read_only = get_admin_details(admin_user, view)
-        assert editable == {
+        fields = get_admin_details(admin_user, view)
+        assert fields == {
             "description",
             "fields",
             "model_name",
@@ -145,46 +114,50 @@ class TestAdminFieldsSuperUser:
             "public",
             "public_slug",
             "query",
-        }
-        assert read_only == {
             "created_time",
             "google_sheets_formula",
             "id",
             "open_view",
             "public_link",
+            "limit",
         }
 
 
 class TestAdminFieldsStaffUser:
     def test_add_page_no_public_fields(self, staff_user, get_admin_details):
-        editable, read_only = get_admin_details(staff_user, None)
-        assert editable == {
+        fields = get_admin_details(staff_user, None)
+        assert fields == {
             "description",
             "fields",
             "model_name",
             "name",
             "owner",
             "query",
+            "created_time",
+            "id",
+            "open_view",
+            "limit",
         }
-        assert read_only == {"created_time", "id", "open_view"}
 
     def test_private_view_no_public_fields(self, staff_user, get_admin_details, view):
-        editable, read_only = get_admin_details(staff_user, view)
-        assert editable == {
+        fields = get_admin_details(staff_user, view)
+        assert fields == {
             "description",
             "fields",
             "model_name",
             "name",
             "owner",
             "query",
+            "created_time",
+            "id",
+            "open_view",
+            "limit",
         }
-        assert read_only == {"created_time", "id", "open_view"}
 
     def test_public_view_readonly(self, staff_user, get_admin_details, view):
         view.public = True
-        editable, read_only = get_admin_details(staff_user, view)
-        assert editable == set()
-        assert read_only == {
+        fields = get_admin_details(staff_user, view)
+        assert fields == {
             "created_time",
             "description",
             "fields",
@@ -194,4 +167,5 @@ class TestAdminFieldsStaffUser:
             "open_view",
             "owner",
             "query",
+            "limit",
         }
