@@ -14,11 +14,14 @@ from data_browser.types import (
     DateTimeType,
     DateType,
     DurationType,
+    HTMLType,
+    IsNullType,
     MonthType,
     NumberChoiceType,
     NumberType,
     StringChoiceType,
     StringType,
+    UnknownType,
     WeekDayType,
     YearType,
 )
@@ -313,7 +316,8 @@ class TestStringType:
         assert StringType.default_lookup == "equals"
 
     def test_format(self):
-        assert StringType.format("bob") == "bob"
+        assert StringType.get_formatter(None)("bob") == "bob"
+        assert StringType.get_formatter(None)(None) is None
 
 
 class TestNumberType:
@@ -328,7 +332,8 @@ class TestNumberType:
         assert NumberType.default_lookup == "equals"
 
     def test_format(self):
-        assert NumberType.format(6) == 6
+        assert NumberType.get_formatter(None)(6) == 6
+        assert NumberType.get_formatter(None)(None) is None
 
 
 class TestYearType:
@@ -344,7 +349,8 @@ class TestYearType:
         assert YearType.default_lookup == "equals"
 
     def test_format(self):
-        assert YearType.format(6) == 6
+        assert YearType.get_formatter(None)(6) == 6
+        assert YearType.get_formatter(None)(None) is None
 
 
 class TestDateTimeType:
@@ -359,15 +365,25 @@ class TestDateTimeType:
     def test_default_lookup(self):
         assert DateTimeType.default_lookup == "equals"
 
-    def test_format(self):
+    @pytest.mark.django_db
+    def test_aware_format(self, settings):
+        settings.USE_TZ = True
         assert (
-            DateTimeType.format(datetime(2020, 5, 19, 8, 42, 16, tzinfo=timezone.utc))
+            DateTimeType.get_formatter(None)(
+                datetime(2020, 5, 19, 8, 42, 16, tzinfo=timezone.utc)
+            )
             == "2020-05-19 08:42:16"
         )
+        assert DateTimeType.get_formatter(None)(None) is None
+
+    @pytest.mark.django_db
+    def test_naive_format(self, settings):
+        settings.USE_TZ = False
         assert (
-            DateTimeType.format(datetime(2020, 5, 19, 8, 42, 16))
+            DateTimeType.get_formatter(None)(datetime(2020, 5, 19, 8, 42, 16))
             == "2020-05-19 08:42:16"
         )
+        assert DateTimeType.get_formatter(None)(None) is None
 
 
 class TestDurationType:
@@ -386,8 +402,12 @@ class TestDurationType:
         assert DurationType.default_lookup == "equals"
 
     def test_format(self):
-        assert DurationType.format(timedelta(days=5, minutes=6)) == "5 days, 0:06:00"
-        assert DurationType.format(timedelta(minutes=6)) == "0:06:00"
+        assert (
+            DurationType.get_formatter(None)(timedelta(days=5, minutes=6))
+            == "5 days, 0:06:00"
+        )
+        assert DurationType.get_formatter(None)(timedelta(minutes=6)) == "0:06:00"
+        assert DurationType.get_formatter(None)(None) is None
 
 
 class TestDateType:
@@ -403,7 +423,8 @@ class TestDateType:
         assert DateType.default_lookup == "equals"
 
     def test_format(self):
-        assert DateType.format(date(2020, 5, 19)) == "2020-05-19"
+        assert DateType.get_formatter(None)(date(2020, 5, 19)) == "2020-05-19"
+        assert DateType.get_formatter(None)(None) is None
 
 
 class TestWeekDayType:
@@ -414,8 +435,9 @@ class TestWeekDayType:
         assert WeekDayType.parse("gt", "Monday") == (None, ANY(str))
 
     def test_format(self):
-        assert WeekDayType.format(1) == "Sunday"
-        assert WeekDayType.format(7) == "Saturday"
+        assert WeekDayType.get_formatter(None)(1) == "Sunday"
+        assert WeekDayType.get_formatter(None)(7) == "Saturday"
+        assert WeekDayType.get_formatter(None)(None) is None
 
     def test_default_lookup(self):
         assert WeekDayType.default_lookup == "equals"
@@ -429,8 +451,9 @@ class TestMonthType:
         assert MonthType.parse("gt", "January") == (None, ANY(str))
 
     def test_format(self):
-        assert MonthType.format(1) == "January"
-        assert MonthType.format(12) == "December"
+        assert MonthType.get_formatter(None)(1) == "January"
+        assert MonthType.get_formatter(None)(12) == "December"
+        assert MonthType.get_formatter(None)(None) is None
 
     def test_default_lookup(self):
         assert MonthType.default_lookup == "equals"
@@ -443,15 +466,54 @@ class TestBooleanType:
         assert BooleanType.parse("equals", "hello") == (None, ANY(str))
         assert BooleanType.parse("pontains", "True") == (None, ANY(str))
 
+    def test_format(self):
+        assert BooleanType.get_formatter(None)(None) is None
+
     def test_default_lookup(self):
         assert BooleanType.default_lookup == "equals"
 
 
 class TestStringChoiceType:
     def test_format(self):
-        assert StringChoiceType.format("b", [("a", "A"), ("b", "B"), ("c", "C")]) == "B"
+        assert (
+            StringChoiceType.get_formatter([("a", "A"), ("b", "B"), ("c", "C")])("b")
+            == "B"
+        )
+        assert (
+            StringChoiceType.get_formatter([("a", "A"), ("b", "B"), ("c", "C")])(None)
+            is None
+        )
+
+    def test_bad_value(self):
+        assert (
+            StringChoiceType.get_formatter([("a", "A"), ("b", "B"), ("c", "C")])("x")
+            == "x"
+        )
 
 
 class TestNumberChoiceType:
     def test_format(self):
-        assert NumberChoiceType.format(2, [(1, "A"), (2, "B"), (3, "C")]) == "B"
+        assert NumberChoiceType.get_formatter([(1, "A"), (2, "B"), (3, "C")])(2) == "B"
+        assert (
+            NumberChoiceType.get_formatter([(1, "A"), (2, "B"), (3, "C")])(None) is None
+        )
+
+    def test_bad_value(self):
+        assert NumberChoiceType.get_formatter([(1, "A"), (2, "B"), (3, "C")])(6) == 6
+
+
+class TestHTMLType:
+    def test_format(self):
+        assert HTMLType.get_formatter(None)(None) is None
+
+
+class TestIsNullType:
+    def test_format(self):
+        assert IsNullType.get_formatter(None)(True) == "IsNull"
+        assert IsNullType.get_formatter(None)(False) == "NotNull"
+        assert IsNullType.get_formatter(None)(None) is None
+
+
+class TestUnknownType:
+    def test_format(self):
+        assert UnknownType.get_formatter(None)(None) is None
